#!BPY

"""
Name: 'Model (.model)...'
Blender: 268
Group: 'Export'
Tip: 'Export current scene to the model format'
"""

__author__ = "Johnny Götz"
__version__= "001"

import bpy as bl
import mathutils as blmath
#import BPyMessages as BMsg
import struct
import math
import time
from time import localtime, strftime

smoothDegree = 0.5


class Mat4x3:
    __slots__ = "vec1", "vec2", "vec3", "vec4"
   
    def __init__(self):
        self.vec1 = [1.0, 0.0, 0.0]
        self.vec2 = [0.0, 1.0, 0.0]
        self.vec3 = [0.0, 0.0, 1.0]
        self.vec4 = [0.0, 0.0, 0.0]
       
    def save(self, file):
        file.writeFloat(self.vec1[0])
        file.writeFloat(self.vec1[1])
        file.writeFloat(self.vec1[2])
   
        file.writeFloat(self.vec2[0])
        file.writeFloat(self.vec2[1])
        file.writeFloat(self.vec2[2])
        
        file.writeFloat(self.vec3[0])
        file.writeFloat(self.vec3[1])
        file.writeFloat(self.vec3[2])
        
        file.writeFloat(self.vec4[0])
        file.writeFloat(self.vec4[1])
        file.writeFloat(self.vec4[2])

class Vertex:
    __slots__ = "position", "normal", "tangent", "bitangent", "color", \
                "uv1","uv2", "uv3", "uv4", "boneIndex", "boneWeights", "baseNormal"
                
    def __init__(self, pos, col = blmath.Vector([0.0, 0.0, 0.0, 0.0]), \
                 uv = blmath.Vector([0.0, 0.0])):
        self.position = blmath.Vector([pos[0], pos[2], -pos[1], 1.0])
        self.normal = blmath.Vector([0.0, 0.0, 0.0])
        self.tangent = blmath.Vector([0.0, 0.0, 0.0])
        self.bitangent = blmath.Vector([0.0, 0.0, 0.0])
        self.color = col
        self.uv1 = uv
        self.uv2 = blmath.Vector([0.0, 0.0])
        self.uv3 = blmath.Vector([0.0, 0.0])
        self.uv4 = blmath.Vector([0.0, 0.0]) 
        self.boneIndex = blmath.Vector([0.0, 0.0, 0.0, 0.0])
        self.boneWeights = blmath.Vector([0.0, 0.0, 0.0, 0.0])
        self.baseNormal = None
 
    def setColor(self, color):
        self.color = [color[0] / 255.0, color[1] / 255.0, 
                      color[2] / 255.0, color[3] / 255.0,]
    
    def setUV(self, uv):
        self.uv1 = uv
        
    def accumulateNormal(self, normal):
        normal.normalize()
        if self.baseNormal == None:
            self.baseNormal = normal;
        self.normal += normal
        
        
    def accumulateTangents(self, uTangent, vTangent):
        self.tangent += uTangent
        self.bitangent += vTangent
        
    def normalize(self):
        self.normal.normalize()
        self.tangent.normalize()
        self.bitangent.normalize()
        
    
        
    def equalColor(self, color):
        if self.color != None and color != None:
            return self.color[0] == color[0] and self.color[1] == color[1] and \
                   self.color[2] == color[2] and self.color[3] == color[3]
        return True
        
    def equalUV(self, uv):
        if self.uv != None and uv != None:
            return self.uv1[0] == uv[0] and self.uv1[1] == uv[1]
        return True
    
    def equalPos(self, pos):
        return self.pos == pos
    
    def equalNor(self, normal):
        global smoothDegree
        normal.normalize()
        return baseNormal * normal >= smoothDegree if self.baseNormal != None else True
        
    def equals(self, vertex, normal):
        return self.equalPos(vertex.pos) and self.euqalNor(normal) and self.euqalUV(vertex.uv)
    
    def save(self, file):
        file.writeFloat(self.position[0])
        file.writeFloat(self.position[1])
        file.writeFloat(self.position[2])
        file.writeFloat(0.0)
        
        file.writeFloat(self.normal[0])
        file.writeFloat(self.normal[1])
        file.writeFloat(self.normal[2])
        file.writeFloat(0.0)
        
        file.writeFloat(self.tangent[0])
        file.writeFloat(self.tangent[1])
        file.writeFloat(self.tangent[2])
        file.writeFloat(0.0)
        
        file.writeFloat(self.bitangent[0])
        file.writeFloat(self.bitangent[1])
        file.writeFloat(self.bitangent[2])
        file.writeFloat(0.0)
        
        file.writeFloat(self.color[0])
        file.writeFloat(self.color[1])
        file.writeFloat(self.color[2])
        file.writeFloat(self.color[3])
        
        file.writeFloat(self.uv1[0])
        file.writeFloat(self.uv1[1])
        file.writeFloat(0.0)
        file.writeFloat(0.0)
        
        file.writeFloat(self.uv1[0])
        file.writeFloat(self.uv1[1])
        file.writeFloat(0.0)
        file.writeFloat(0.0)
        
        file.writeFloat(self.uv1[0])
        file.writeFloat(self.uv1[1])
        file.writeFloat(0.0)
        file.writeFloat(0.0)
        
        file.writeFloat(self.uv1[0])
        file.writeFloat(self.uv1[1])
        file.writeFloat(0.0)
        file.writeFloat(0.0)
        
        file.writeFloat(self.boneIndex[0])
        file.writeFloat(self.boneIndex[1])
        file.writeFloat(self.boneIndex[2])
        file.writeFloat(self.boneIndex[3])
        
        file.writeFloat(self.boneWeights[0])
        file.writeFloat(self.boneWeights[1])
        file.writeFloat(self.boneWeights[2])
        file.writeFloat(self.boneWeights[3])
        

class MeshSegment:
    __slots__ = "material", "primitiveType", "firstIndex", "count"
    
    def __init__(self, firstIdx, count):
        self.material = "NONE"
        self.primitiveType = 1
        self.firstIndex = firstIdx
        self.count = count
        
    def save(self, file):
        file.writeString(self.material)
        file.writeInt(self.primitiveType)
        file.writeInt(self.firstIndex)
        file.writeInt(self.count)
               
class MeshData:
    __slots__ = "magic", "version", "vertices", "indices", "segments", \
                "boundCenter", "boundRadius", "boundMin", "boundMax", "blMesh"
                
    def __init__(self, blMesh):
        self.magic = "MESH"
        self.blMesh = blMesh
        self.version = 1
        self.boundCenter = blmath.Vector([0.0, 0.0, 0.0])
        self.boundRadius = 0.0
        self.boundMin = blmath.Vector([0.0, 0.0, 0.0])
        self.boundMax = blmath.Vector([0.0, 0.0, 0.0])
        self.vertices = []
        self.indices = []
        self.segments = []
        for vertex in blMesh.vertices:
            self.addVertex(vertex)
        for polygon in blMesh.polygons:
            #self.addFace(face)
            self.addPolygon(polygon)
        self.segments.append(MeshSegment(0, len(self.indices)))
        print (len(self.indices))     
        
        
    def addVertex(self, vertex):
        self.vertices.append(Vertex(vertex.co))
        
    def addIndex(self, index):
        self.indices.append(index)
   
    def addPolygon(self, polygon):
        numVerts = len(polygon.vertices)
        uvs = [blmath.Vector([0.0, 0.0]), blmath.Vector([0.0, 0.0]), 
               blmath.Vector([0.0, 0.0]), blmath.Vector([0.0, 0.0])]
        index = [None, None, None, None]
        uTangent = blmath.Vector([0.0, 0.0, 1.0])
        vTangent = blmath.Vector([0.0, 0.0, 1.0])
        
        verts = []
        for i in range(3):
            verts.append(self.vertices[polygon.vertices[i]])
            self.addIndex(polygon.vertices[i])
            uvs[i] = self.blMesh.uv_layers[0].data[i].uv
            
        uTangent, vTangent = self.computeTangent(verts[0].position, verts[1].position,
                                    verts[2].position, 
                                    uvs[0], uvs[1], uvs[2])
            
        normal = (verts[2].position - verts[1].position).cross(
                                  verts[0].position - verts[1].position)
                       
        for i in range(3):
            verts[i].accumulateNormal(normal)
            verts[i].accumulateTangents(uTangent, vTangent)
            verts[i].setUV(uvs[i])
     
            
        if numVerts > 3:
            verts.append(self.vertices[polygon.vertices[3]])
            uvs[3] = self.blMesh.uv_layers[0].data[3].uv 
            uTangent, vTangent = self.computeTangent(verts[0].position, verts[2].position,
                                    verts[3].position,
                                    uvs[0], uvs[2], uvs[3])
            normal = (verts[0].position - verts[3].position).cross(
                                      verts[2].position - verts[3].position)
                              
            for i in [0,2,3]:
                verts[i].accumulateNormal(normal)
                verts[i].accumulateTangents(uTangent, vTangent)
                verts[i].setUV(uvs[i])
                self.addIndex(polygon.vertices[i])
   
    def addFace(self, face):
        numVerts = len(face.vertices)
        uvs = [blmath.Vector([0.0, 0.0]), blmath.Vector([0.0, 0.0]), 
               blmath.Vector([0.0, 0.0]), blmath.Vector([0.0, 0.0])]
        index = [None, None, None, None]
        uTangent = blmath.Vector([0.0, 0.0, 1.0])
        vTangent = blmath.Vector([0.0, 0.0, 1.0])
        
        if self.blMesh.vertex_colors:
            for i in range(0, numVerts):
                colors[i] = face.col[i]
        if self.blMesh.uv_layers:
            for i in range(0, numVerts):
                uvs[i] = face.uv[i]
            uTangent, vTangent = self.computeTangent(face.verts[0].co, face.verts[1].co,
                                                     face.verts[2].co, uvs[0], uvs[1], uvs[2])

            normal = blmath.CrossVecs(face.verts[2].co - face.verts[1].co, 
                                      face.verts[0].co - face.verts[1].co)
                                      
            index[0] = self.addFaceVertex(face.verts[0], normal, uTangent, vTangent, uvs[0], colors[0])
            index[1] = self.addFaceVertex(face.verts[1], normal, uTangent, vTangent, uvs[1], colors[1])
            index[2] = self.addFaceVertex(face.verts[2], normal, uTangent, vTangent, uvs[2], colors[2])
            self.addIndex(index[0])
            self.addIndex(index[1])
            self.addIndex(index[2])
            
            if numVertes >= 4:
                normal = blmath.CrossVecs(face.verts[0].co - face.verts[3].co, 
                                          face.verts[2].co - face.verts[3].co)
                index[3] = self.addFaceVertex(face.verts[3], normal, uTangent, vTangent, 
                                              uvs[3], colors[3])
                self.addIndex(index[0])
                self.addIndex(index[2])
                self.addIndex(index[3])
                                                     

    def addFaceVertex(self, vertex, normal, uTangent, vTangent, uv, color):
        index = -1
        bones = [0.0, 0.0, 0.0, 0.0]
        weights = [0.0, 0.0, 0.0, 0.0]
        influences = self.blMesh.getVertexInfluences(vertex.index)
        
        #TODO: bones
        #if influences != None:
        
        foundVertex = Vertex(vertex.co, color, uv)
        
        if self.vertices[vertex.index].equals(foundVertex, normal) == TRUE:
            foundVertex = self.vertices[vertex.index]
            index = vertex.index
        
        if index == -1:
            for i in range(len(self.blMesh.verts), len(self.vertices)):
                if self.vertices[i].equals(foundVertex, normal) == True:
                    foundVertex = self.vertices[i]
                    index = i
                    break
       
        foundVertex.accumulateNormal(normal)
        foundVertex.accumulateTangents(uTangent, vTangent)
        foundVertex.setUV(uv)
        foundVertex.setColor(color)
        
        #TODO:
        #bones, weight
        
        
        if index == -1:
            index = len(self.vertices)
            self.vertices.append(foundVertex)
            
        return index
                        

    def computeTangent(self, v0, v1, v2, uv0, uv1, uv2):
        d1 = v1 - v0
        d2 = v2 - v0
        
        t1 = uv1 - uv0
        t2 = uv2 - uv0
        
        t = (t1[0] * t2[1] - t1[1] * t2[0])
        if t != 0:
            r = 1.0 / t;
        else:
            r = 1.0
        uTangent = blmath.Vector([ (t2[1] * d1.x - t1[1] * d2.x) * r, 
                                   (t2[1] * d1.y - t1[1] * d2.y) * r,
                                   (t2[1] * d1.z - t1[1] * d2.z) * r ])
        vTangent = blmath.Vector([ (t1[0] * d2.x - t2[0] * d1.x) * r,
                                   (t1[0] * d2.y - t2[0] * d1.y) * r,
                                   (t1[0] * d2.z - t2[0] * d1.z) * r])
                                   
        return uTangent, vTangent
    
    def save(self, file):
        file.writeFourCC(self.magic)
        file.writeInt(self.version)
        file.writeInt(len(self.vertices))
        for i in range(len(self.vertices)):
            self.vertices[i].save(file)
        file.writeInt(len(self.indices))
        for i in range(len(self.indices)):
            file.writeUInt(self.indices[i])
        file.writeInt(len(self.segments))
        for i in range(len(self.segments)):
            self.segments[i].save(file)
        file.writeVec3(self.boundCenter)
        file.writeFloat(self.boundRadius)
        file.writeVec3(self.boundMin)
        file.writeVec3(self.boundMax)
        

class MeshEntity:
    __slots__ = "meshData", "numBones", "bones", "emissiveColor", "castShadow"
    
    def __init__(self, blMesh):
        self.numBones = 0;
        self.bones = [];
        self.emissiveColor = blmath.Vector([0.0, 0.0, 0.0]);
        self.castShadow = 0;
        self.meshData = MeshData(blMesh);
        
        
    def save(self, file):
        self.meshData.save(file)
        file.writeInt(0)
        file.writeVec3([0,0,0])
        file.writeUChar(0)
        

class Node:
    __slots__ = "blName", "blMesh", "name", "parent", "type", "parentTransform", \
                "meshEntity"
    
    def __init__(self, meshData, parent):
        self.name = "ModelFloor"
        self.blMesh = meshData
        self.blName = self.blMesh.name
        self.parent = parent
        self.parentTransform = Mat4x3()
        self.type = 0;
        self.meshEntity = MeshEntity(self.blMesh)
        
    def getBlName(self):
        return self.blName
    
    
    def save(self, file):
        file.writeString(self.name)
        self.parentTransform.save(file)
        file.writeInt(self.parent)
        file.writeInt(self.type)
        self.meshEntity.save(file)
        

class Model:
    __slots__ = "magic", "version", "nodes"
    
    def __init__(self, scene):
        self.magic = "MDL1"
        self.version = 1
        self.nodes = []
        for meshData in bl.data.meshes:
            #print (object.type)
            #if object.type != "MESH":
            #    continue
            #meshData = object.getData(False, True)
            
            #print("bla")
            node = Node(meshData, len(self.nodes) - 1);
            self.nodes.append(node)
        
    def getMeshID(self, name):
        for i in range(len(self.nodes)):
            if name == self.nodes[i].getBlName():
                return i;    


    def save(self, file):
        file.writeFourCC(self.magic)
        file.writeInt(self.version)
        file.writeInt(len(self.nodes))
        for i in range(len(self.nodes)):
            self.nodes[i].save(file)
        
        
class File:
    __slots__ = "outFile"
    
    
    def __init__(self, filename):
        if not filename.lower().endswith('.model'):
            filename += '.model'
       
        self.outFile = open(filename, "wb")
        
    def close(self):
        if self.outFile:
            self.outFile.close()
            self.outFile = None
            
    def writeInt(self, i):
        self.outFile.write(struct.pack("<i", i))
        
    def writeUInt(self, i):
        self.outFile.write(struct.pack("<I", i))
        
    def writeLong(self, i):
        self.outFile.write(struct.pack("<l", i))
        
    def writeULong(self, i):
        self.outFile.write(struct.pack("<L", i))
        
    def writeShort(self, s):
        self.outFile.write(struct.pack("<h", s))
    
    def writeUShort(self, s):
        self.outFile.write(struct.pack("<H", s))
        
    def writeFloat(self, f):
        self.outFile.write(struct.pack("<f", f))
        
    def writeDouble(self, d):
        self.outFile.write(struct.pack("<d", f))
   
    def writeChar(self, c):
        self.outFile.write(struct.pack("<c", c))
   
    def writeUChar(self, c):
        self.outFile.write(struct.pack("<B", c)) 
        
    def writeBool(self, b):
        self.outFile.write(struct.pack("<B", 1 if b else 0))
        
    def writeExpand(self, o, l, c = "\0"):
        s = len(o)
        self.outFile.write(o)
        for i in range(l - s):
            self.writeChar(c)            
            
    def writeFourCC(self, magic):
        for i in range(4):
            self.writeChar(magic[i].encode('ascii'))   
            
    def writeString(self, string):
        self.writeInt(len(string))
        for i in range(len(string)):
            self.writeChar(string[i].encode('ascii'))  
            
    def writeVec3(self, vec3):
        for i in range(3):
            self.writeFloat(vec3[i])
            
    def writeMat4x3(self, matrix):
        for i in range(4):
            self.writeVec3(matrix[i])   
  
    def write(self, o):
        self.outFile.write(o)

def save(filename):
    print (strftime("%a, %d %b %Y %H:%M:%S", localtime()))
    
    scene = bl.data.scenes.get("Scene", None)
    model = Model(scene)
    
    if filename != None:
        outFile = File(filename)
        model.save(outFile)
    
    #bl.Redraw()
    
    print ("Done!")



if __name__ == '__main__':
    #bl.Window.FileSelector(save, "Export model", bl.sys.makename(ext='.model')) 
    save('/Users/johnny/code/galaxycode/wizzy/wizzy/model/ceiling.model')