#version 150

#define MAX_LIGHTS 4

uniform vec4 lightPosition;
uniform vec3 lightColor;
uniform float lightPower;

uniform sampler2D colorTexture;
uniform sampler2D normalTexture;
uniform sampler2D specularTexture;
uniform samplerCube shadowMap;


in vec4 vTexCoord;

//world space
in vec3 vPosition_world;

//camera space
in vec3 vEyeVec_camera;
in vec3 vLightDirection_camera;
//tangent space
in vec3 vLightDirection_tangent;
in vec3 vEyeVec_tangent;

in vec4 vLightDistance_world;

in vec4 vShadowCoord;

out vec4 fragmentColor;


float VectorToDepthValue(vec3 lightVector)
{
	vec3 absLightVector = abs(lightVector);
	float LocalZComp = max(absLightVector.x, max(absLightVector.y, absLightVector.z));
	
	const float f = 1024.0;
	const float n = 0.125f;

	float NormZComp = (f + n) / (f - n) - (2 * f * n) / (f - n) / LocalZComp;
	return (NormZComp + 1.0) * 0.5;
}

void main(void)
{
	float bias = 0.0001;
	float visibility = 1.0;
	float shadowDepth = texture(shadowMap, -vLightDistance_world);
	if(shadowDepth + bias < VectorToDepthValue(-vLightDistance_world))
		visibility = 0.4; 
	//if(shadowDepth + bias < length(vLightDistance_world.xyz))
	//	visibility = 0.4;

    vec3 diffuseColor = texture(colorTexture, vTexCoord.xy).rgb;
    vec3 specularColor = texture(specularTexture, vTexCoord.xy).rgb * 0.3;
    vec3 normal_tangent = normalize(texture(normalTexture, vTexCoord.xy).rgb * 2.0f - 1.0f);
    
    float distance = length(vLightDistance_world.xyz);
    
    vec3 n = normal_tangent;
    vec3 l = normalize(vLightDirection_tangent);
    
    float cosTheta = clamp( dot( n, l), 0.3, 1);
    
    vec3 E = normalize(vEyeVec_tangent);
    vec3 R = reflect(-l, n);
    
    float cosAlpha = clamp( dot(E, R), 0, 1);
    
    fragmentColor = vec4(diffuseColor * vec3(0.1, 0.1, 0.1) +
                         visibility * diffuseColor * lightColor.xyz * lightPower * cosTheta / (distance) +
                         visibility * specularColor * lightColor.xyz * lightPower * pow(cosAlpha, 5) / (distance),
                         1.0);
    
    
	//fragmentColor = vec4(cosTheta * diffuseColor, 1.0f);
	//fragmentColor = vec4(E, 1.0f);
	//fragmentColor = vec4(vLightDistance_world.xyz, 1.0f);
	//fragmentColor = vec4(specularColor, 1.0f);
    //fragmentColor = vec4(normal_tangent, 1.0f);
    //fragmentColor = vec4(specularColor, 1.0f);
    //fragmentColor = vec4(0.0, 1.0, 0.0, 1.0);
	//fragmentColor = shadowDepth;
}

