#version 150

#define MAX_LIGHTS 4

uniform mat4 modelMatrix;
uniform mat4 viewMatrix;
uniform mat4 projectionMatrix;

uniform vec4[MAX_LIGHTS] lightPosition;
uniform int numLights;

in vec4 position;
in vec4 normal;
in vec4 tangent;
in vec4 binormal;
in vec4 color;
in vec4 uv1;
in vec4 uv2;
in vec4 uv3;
in vec4 uv4;
in vec4 boneIndex;
in vec4 boneWeight;

out vec4 vTexCoord;

//world space
out vec3 vPosition_world;
out vec4 vLightDistance_world[MAX_LIGHTS];

//camera space
out vec3 vEyeVec_camera;
out vec3 vLightDirection_camera[MAX_LIGHTS];

//tangent space
out vec3 vLightDirection_tangent[MAX_LIGHTS];
out vec3 vEyeVec_tangent[MAX_LIGHTS];

void main(void)
{
    vec4 posWorldSpace = modelMatrix * vec4(position.xyz, 1.0f);
    vec4 posCameraSpace = viewMatrix * posWorldSpace;
    gl_Position = projectionMatrix * viewMatrix * modelMatrix * vec4(position.xyz, 1.0f);
    
    vPosition_world = posWorldSpace.xyz;
    
    vEyeVec_camera = -posCameraSpace.xyz;
    
	for(int i = 0; i < MAX_LIGHTS; i++) {
		if(i <= numLights) {
			vec3 lightPos_camera = (viewMatrix * lightPosition[i]).xyz;
			vLightDirection_camera[i] = lightPos_camera + vEyeVec_camera;
    
			mat3 modelView = mat3(viewMatrix) * mat3(modelMatrix);
			mat3 tbnMatrix = transpose(mat3((modelView * tangent.xyz),
											(modelView * binormal.xyz),
											(modelView * normal.xyz)));
    
			vLightDirection_tangent[i] = tbnMatrix * vLightDirection_camera[i];
			vEyeVec_tangent[i] = tbnMatrix * vEyeVec_camera;
    
			vLightDistance_world[i] = vec4(lightPosition[i].xyz, 1.0f) - posWorldSpace;
		}
	}

	vTexCoord = uv1;
}

