#version 150

#define MAX_LIGHTS 4

uniform mat4 modelMatrix;
uniform mat4 viewMatrix;
uniform mat4 projectionMatrix;

uniform mat3 normalMatrix;
uniform vec4 lightPosition;
uniform vec3 lightDirection;

//uniform mat4 depthBiasMVP;

in vec4 position;
in vec4 normal;
in vec4 tangent;
in vec4 binormal;
in vec4 color;
in vec4 uv1;
in vec4 uv2;
in vec4 uv3;
in vec4 uv4;
in vec4 boneIndex;
in vec4 boneWeight;

out vec4 vTexCoord;

//world space
out vec3 vPosition_world;

//camera space
out vec3 vEyeVec_camera;
out vec3 vLightDirection_camera;
//tangent space
out vec3 vLightDirection_tangent;
out vec3 vEyeVec_tangent;

out vec4 vLightDistance_world;

out vec4 vShadowCoord;

void main(void)
{
    vec4 posWorldSpace = modelMatrix * vec4(position.xyz, 1.0f);
    vec4 posCameraSpace = viewMatrix * posWorldSpace;
    gl_Position = projectionMatrix * viewMatrix * modelMatrix * vec4(position.xyz, 1.0f);

	//vShadowCoord = depthBiasMVP * vec4(position.xyz, 1.0f);
	//vShadowCoord = posWorldSpace - lightPosition;
    
    vPosition_world = posWorldSpace.xyz;
    
    vEyeVec_camera = -posCameraSpace.xyz;
    
    vec3 lightPos_camera = (viewMatrix * modelMatrix * lightPosition).xyz;
    vLightDirection_camera = lightPos_camera + vEyeVec_camera;
    
    vTexCoord = uv1;
    
    mat3 modelView = mat3(viewMatrix) * mat3(modelMatrix);
    mat3 tbnMatrix = transpose(mat3((modelView * normalize(tangent.xyz)),
                                    (modelView * normalize(binormal.xyz)),
                                    (modelView * normalize(normal.xyz))));
    //tbnMatrix = normalMatrix * tbnMatrix;
    
    
    vLightDirection_tangent = tbnMatrix * vLightDirection_camera;
    vEyeVec_tangent = tbnMatrix * vEyeVec_camera;
    
    vLightDistance_world = vec4(lightPosition.xyz, 1.0f) - posWorldSpace;

	
}

